#! python3
# -*- encoding: utf-8 -*-

__title__ = "rtsf"
__short_desc__ = "a test service framework"
__version__ = "3.0.0"
__autor__ = u"RockFeng0"
__author_email__ = "281589463@qq.com"
__license__ = 'MIT'

HOME_PAGE = "https://github.com/RockFeng0/rtsf"
